using System.Collections;
using API.Controllers;
using API.Service;
using DbContextLib;
using DbDataManager.Service;
using Dto;
using Microsoft.AspNetCore.Http.HttpResults;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Logging.Abstractions;
using Moq;
using Shared;
using TestAPI.Extensions;

namespace TestAPI
{
    public class UserUnitTest
    {
        private readonly Mock<IUserService<UserDTO>> userService;
        public UserUnitTest()
        {
            userService = new Mock<IUserService<UserDTO>>();
        }

        [Fact]
        public void GetUserListSuccess()
        {
            var userList = GetUsersData();
            userService.Setup(x => x.GetUsers(1 ,4,0))
                .Returns(userList);
            var UsersController = new UsersController(new NullLogger<UsersController>(), userService.Object);

            var userResult = UsersController.GetUsers(1,4, 0);

            if (userResult is OkObjectResult okObjectResult)
            {
                var valeur = okObjectResult.Value;

                Assert.NotNull(valeur); 
                Assert.Equal(GetUsersData().ToString(), valeur.ToString());
                Assert.True(userList.SequenceEqual(valeur as IEnumerable<UserDTO>,new UserIdEqualityComparer()));
                
            }

        }

        [Fact]
        public void GetUserListFail()
        { 
            userService.Setup(x => x.GetUsers(1, 4,0))
                .Returns(new List<UserDTO>());
            var UsersController = new UsersController(new NullLogger<UsersController>(), userService.Object);

            var userResult = UsersController.GetUsers(2,3, 0);

            if (userResult is StatusCodeResult statusCodeResult && statusCodeResult.StatusCode == 204)

            {
                
                Assert.IsNotType<OkObjectResult>(userResult);
            }

        }


        [Fact]
        public void GetUserIdSuccess()
        {
            var userList = GetUsersData();
            userService.Setup(x => x.GetUserById(1))
                .Returns(userList[1]);
            var UsersController = new UsersController(new NullLogger<UsersController>(), userService.Object);

            var userResult = UsersController.GetUserById(1);
            if (userResult is OkObjectResult okObjectResult)
            {
                UserDTO valeur = okObjectResult.Value as UserDTO;

                Assert.NotNull(valeur);
                Assert.Equal("Leuser",valeur.Username);
                Assert.Equal("motdepasse", valeur.Password);
                Assert.Equal("deuxadresse@gmail.com", valeur.Email);
                Assert.IsType<UserDTO>(valeur);
                Assert.Contains(valeur, userList);
            }

        }

        [Fact]
        public void GetUserIdFail()
        {
            var userList = GetUsersData();
            userService.Setup(x => x.GetUserById(1))
                .Returns(userList[1]);
            var UsersController = new UsersController(new NullLogger<UsersController>(), userService.Object);

            var userResult = UsersController.GetUserById(100);
            if (userResult is NotFoundObjectResult NFObjectResult)
            {
                var valeur = NFObjectResult.Value;

                Assert.NotNull(valeur);
                Assert.IsNotType<UserDTO>(valeur);
                Assert.DoesNotContain(valeur, userList);
            }

        }


        [Fact]
        public void GetUserUsernameSuccess()
        {
            var userList = GetUsersData();
            userService.Setup(x => x.GetUserByUsername("Useruser"))
                .Returns(userList[0]);
            var UsersController = new UsersController(new NullLogger<UsersController>(), userService.Object);

            var userResult = UsersController.GetUserByUsername("Useruser");
            if (userResult is OkObjectResult okObjectResult)
            {
                UserDTO valeur = okObjectResult.Value as UserDTO;

                Assert.NotNull(valeur);
                Assert.IsType<UserDTO>(valeur);
                Assert.Contains(valeur, userList);
                Assert.Equal("adressemail@gmail.com", valeur.Email);
                Assert.Equal("Useruser", valeur.Username);
                Assert.Equal("motdepasse", valeur.Password);
                Assert.True(valeur.IsAdmin);
            }

        }


        [Fact]
        public void GetUserUsernameFail()
        {
            var userList = GetUsersData();
            userService.Setup(x => x.GetUserByUsername("Useruser"))
                .Returns(userList[0]);
            var UsersController = new UsersController(new NullLogger<UsersController>(), userService.Object);

            var userResult = UsersController.GetUserByUsername("GYIIieihhh");

            if (userResult is NotFoundObjectResult NFObjectResult)
            {
                var valeur = NFObjectResult.Value;

                Assert.NotNull(valeur);
                Assert.IsNotType<UserDTO>(valeur);
                Assert.DoesNotContain(valeur, userList);
                Assert.False(userList == valeur);
            }
        }






        [Fact]
        public void DeleteUserSuccess()
        {
            userService.Setup(x => x.DeleteUser(1))
                .Returns(true);
            var UsersController = new UsersController(new NullLogger<UsersController>(), userService.Object);

            var userResult = UsersController.DeleteUser(1);
            if (userResult is OkObjectResult okObjectResult)
            {
                bool valeur = (bool)okObjectResult.Value;

                Assert.True(valeur);
            }

        }


        [Fact]
        public void DeleteUserFail()
        {
            userService.Setup(x => x.DeleteUser(1))
                .Returns(true);
            var UsersController = new UsersController(new NullLogger<UsersController>(), userService.Object);

            var userResult = UsersController.DeleteUser(100);
            if (userResult is NotFoundObjectResult NFObjectResult)
            {
                Assert.Null(NFObjectResult.Value);
                Assert.IsNotType<bool>(NFObjectResult.Value);
            }
        }


        [Fact]
        public void CreateUserSuccess()
        {
            userService.Setup(x => x.CreateUser("Nom","Passssss","heudfk@hdye.com",true))
                .Returns(new UserDTO("Nom","Passssss", "heudfk@hdye.com",true));
            var UsersController = new UsersController(new NullLogger<UsersController>(), userService.Object);

            var userResult = UsersController.CreateUser(new UserDTO("Nom", "Passssss", "heudfk@hdye.com", true));
            if (userResult is CreatedResult createdObjectResult)
            {
                UserDTO valeur = createdObjectResult.Value as UserDTO;

                Assert.NotNull(valeur);
                Assert.Equal("Nom", valeur.Username);
                Assert.Equal("Passssss",valeur.Password);
                Assert.Equal("heudfk@hdye.com",valeur.Email);
                Assert.True(valeur.IsAdmin);
            }

        }

        [Fact]
        public void CreateUserFail()
        {
            var userList = GetUsersData();
            userService.Setup(x => x.CreateUser("Nom", "Passssss", "heudfk@hdye.com", true))
                .Returns(new UserDTO("Nom", "Passssss", "heudfk@hdye.com", true));
            var UsersController = new UsersController(new NullLogger<UsersController>(), userService.Object);

            var userResult = UsersController.CreateUser(new UserDTO(null, "Passssss", "heudfk@hdye.com", true));

            if (userResult is BadRequestResult BDObjectResult)
            {

                Assert.Equal(400, BDObjectResult.StatusCode);
            }
        }


        [Fact]
        public void UpdateUserSuccess()
        {
            userService.Setup(x => x.UpdateUser(1, new UserDTO("Nom", "Passssss", "heudfk@hdye.com", true)))
                .Returns(new UserDTO("Nom", "Passssss", "heudfk@hdye.com", true));
            var UsersController = new UsersController(new NullLogger<UsersController>(), userService.Object);

            var userResult = UsersController.UpdateUser(1, new UserDTO("Nom", "Passssss", "heudfk@hdye.com", true));
            if (userResult is OkObjectResult okObjectResult)
            {
                UserDTO valeur = okObjectResult.Value as UserDTO;

                Assert.NotNull(valeur);
                Assert.Equal("Nom", valeur.Username);
                Assert.Equal("Passssss", valeur.Password);
                Assert.Equal("heudfk@hdye.com", valeur.Email);
                Assert.True(valeur.IsAdmin);
            }

        }

        [Fact]
        public void UpdateUserFail()
        {
            userService.Setup(x => x.UpdateUser(1, new UserDTO(1,"Nom", "Passssss", "heudfk@hdye.com", true)))
                .Returns(new UserDTO("Nom", "Passssss", "heudfk@hdye.com", true));
            var UsersController = new UsersController(new NullLogger<UsersController>(), userService.Object);

            var userResult = UsersController.UpdateUser(1, new UserDTO(2,"Nom", "Passssss", "heudfk@hdye.com", true));

            if (userResult is BadRequestResult BDObjectResult)
            {

                Assert.Equal(400,BDObjectResult.StatusCode);
            }
        }



        



        private List<UserDTO> GetUsersData()
        {
            List<UserDTO> usersData = new List<UserDTO>(4)
            {
                new (
                    0,
                    "Useruser",
                    "motdepasse",
                    "adressemail@gmail.com" ,
                    
                    true
                ),
                new
                (
                    1,
                    "Leuser",
                    "motdepasse",
                    "deuxadresse@gmail.com" ,

                    false
                ),
                new
                (
                    2,
                    "gygyggyg",
                    "ennodlavehc",
                    "thirdadress@gmail.com" ,

                    false
                ),
                new
                (
                    "ferferf",
                    "h_nh_78",
                    "fourthadress@gmail.com" ,

                    false
                ),
            };
            return usersData;
        }
    }
}